@@ -1,6 +1,8 @@
+import enum
 import logging
+import re
+
 import pymysql
-import sqlparse
 from .packages import special
 from pymysql.constants import FIELD_TYPE
 from pymysql.converters import (convert_datetime,
@@ -18,17 +20,73 @@ FIELD_TYPES.update({
     FIELD_TYPE.NULL: type(None)
 })
 
+
+ERROR_CODE_ACCESS_DENIED = 1045
+
+
+class ServerSpecies(enum.Enum):
+    MySQL = 'MySQL'
+    MariaDB = 'MariaDB'
+    Percona = 'Percona'
+    TiDB = 'TiDB'
+    Unknown = 'MySQL'
+
+
+class ServerInfo:
+    def __init__(self, species, version_str):
+        self.species = species
+        self.version_str = version_str
+        self.version = self.calc_mysql_version_value(version_str)
+
+    @staticmethod
+    def calc_mysql_version_value(version_str) -> int:
+        if not version_str or not isinstance(version_str, str):
+            return 0
+        try:
+            major, minor, patch = version_str.split('.')
+        except ValueError:
+            return 0
+        else:
+            return int(major) * 10_000 + int(minor) * 100 + int(patch)
+
+    @classmethod
+    def from_version_string(cls, version_string):
+        if not version_string:
+            return cls(ServerSpecies.Unknown, '')
+
+        re_species = (
+            (r'(?P<version>[0-9\.]+)-MariaDB', ServerSpecies.MariaDB),
+            (r'[0-9\.]*-TiDB-v(?P<version>[0-9\.]+)-?(?P<comment>[a-z0-9\-]*)', ServerSpecies.TiDB),
+            (r'(?P<version>[0-9\.]+)[a-z0-9]*-(?P<comment>[0-9]+$)',
+             ServerSpecies.Percona),
+            (r'(?P<version>[0-9\.]+)[a-z0-9]*-(?P<comment>[A-Za-z0-9_]+)',
+             ServerSpecies.MySQL),
+        )
+        for regexp, species in re_species:
+            match = re.search(regexp, version_string)
+            if match is not None:
+                parsed_version = match.group('version')
+                detected_species = species
+                break
+        else:
+            detected_species = ServerSpecies.Unknown
+            parsed_version = ''
+
+        return cls(detected_species, parsed_version)
+
+    def __str__(self):
+        if self.species:
+            return f'{self.species.value} {self.version_str}'
+        else:
+            return self.version_str
+
+
 class SQLExecute(object):
 
     databases_query = '''SHOW DATABASES'''
 
     tables_query = '''SHOW TABLES'''
 
-    version_query = '''SELECT @@VERSION'''
-
-    version_comment_query = '''SELECT @@VERSION_COMMENT'''
-    version_comment_query_mysql4 = '''SHOW VARIABLES LIKE "version_comment"'''
-
     show_candidates_query = '''SELECT name from mysql.help_topic WHERE name like "SHOW %"'''
 
     users_query = '''SELECT CONCAT("'", user, "'@'",host,"'") FROM mysql.user'''
@@ -52,7 +110,7 @@ class SQLExecute(object):
         self.charset = charset
         self.local_infile = local_infile
         self.ssl = ssl
-        self._server_type = None
+        self.server_info = None
         self.connection_id = None
         self.ssh_user = ssh_user
         self.ssh_host = ssh_host
@@ -118,11 +176,15 @@ class SQLExecute(object):
         if init_command and len(list(special.split_queries(init_command))) > 1:
             client_flag |= pymysql.constants.CLIENT.MULTI_STATEMENTS
 
+        ssl_context = None
+        if ssl:
+            ssl_context = self._create_ssl_ctx(ssl)
+
         conn = pymysql.connect(
             database=db, user=user, password=password, host=host, port=port,
             unix_socket=socket, use_unicode=True, charset=charset,
             autocommit=True, client_flag=client_flag,
-            local_infile=local_infile, conv=conv, ssl=ssl, program_name="mycli",
+            local_infile=local_infile, conv=conv, ssl=ssl_context, program_name="mycli",
             defer_connect=defer_connect, init_command=init_command
         )
 
@@ -157,6 +219,7 @@ class SQLExecute(object):
         self.init_command = init_command
         # retrieve connection id
         self.reset_connection_id()
+        self.server_info = ServerInfo.from_version_string(conn.server_version)
 
     def run(self, statement):
         """Execute the sql in the database and return the results. The results
@@ -273,37 +336,6 @@ class SQLExecute(object):
                 for row in cur:
                     yield row
 
-    def server_type(self):
-        if self._server_type:
-            return self._server_type
-        with self.conn.cursor() as cur:
-            _logger.debug('Version Query. sql: %r', self.version_query)
-            cur.execute(self.version_query)
-            version = cur.fetchone()[0]
-            if version[0] == '4':
-                _logger.debug('Version Comment. sql: %r',
-                              self.version_comment_query_mysql4)
-                cur.execute(self.version_comment_query_mysql4)
-                version_comment = cur.fetchone()[1].lower()
-                if isinstance(version_comment, bytes):
-                    # with python3 this query returns bytes
-                    version_comment = version_comment.decode('utf-8')
-            else:
-                _logger.debug('Version Comment. sql: %r',
-                              self.version_comment_query)
-                cur.execute(self.version_comment_query)
-                version_comment = cur.fetchone()[0].lower()
-
-        if 'mariadb' in version_comment:
-            product_type = 'mariadb'
-        elif 'percona' in version_comment:
-            product_type = 'percona'
-        else:
-            product_type = 'mysql'
-
-        self._server_type = (product_type, version)
-        return self._server_type
-
     def get_connection_id(self):
         if not self.connection_id:
             self.reset_connection_id()
@@ -312,11 +344,54 @@ class SQLExecute(object):
     def reset_connection_id(self):
         # Remember current connection id
         _logger.debug('Get current connection id')
-        res = self.run('select connection_id()')
-        for title, cur, headers, status in res:
-            self.connection_id = cur.fetchone()[0]
-        _logger.debug('Current connection id: %s', self.connection_id)
+        try:
+            res = self.run('select connection_id()')
+            for title, cur, headers, status in res:
+                self.connection_id = cur.fetchone()[0]
+        except Exception as e:
+            # See #1054
+            self.connection_id = -1
+            _logger.error('Failed to get connection id: %s', e)
+        else:
+            _logger.debug('Current connection id: %s', self.connection_id)
 
     def change_db(self, db):
         self.conn.select_db(db)
         self.dbname = db
+
+    def _create_ssl_ctx(self, sslp):
+        import ssl
+
+        ca = sslp.get("ca")
+        capath = sslp.get("capath")
+        hasnoca = ca is None and capath is None
+        ctx = ssl.create_default_context(cafile=ca, capath=capath)
+        ctx.check_hostname = not hasnoca and sslp.get("check_hostname", True)
+        ctx.verify_mode = ssl.CERT_NONE if hasnoca else ssl.CERT_REQUIRED
+        if "cert" in sslp:
+            ctx.load_cert_chain(sslp["cert"], keyfile=sslp.get("key"))
+        if "cipher" in sslp:
+            ctx.set_ciphers(sslp["cipher"])
+
+        # raise this default to v1.1 or v1.2?
+        ctx.minimum_version = ssl.TLSVersion.TLSv1
+
+        if "tls_version" in sslp:
+            tls_version = sslp["tls_version"]
+
+            if tls_version == "TLSv1":
+                ctx.minimum_version = ssl.TLSVersion.TLSv1
+                ctx.maximum_version = ssl.TLSVersion.TLSv1
+            elif tls_version == "TLSv1.1":
+                ctx.minimum_version = ssl.TLSVersion.TLSv1_1
+                ctx.maximum_version = ssl.TLSVersion.TLSv1_1
+            elif tls_version == "TLSv1.2":
+                ctx.minimum_version = ssl.TLSVersion.TLSv1_2
+                ctx.maximum_version = ssl.TLSVersion.TLSv1_2
+            elif tls_version == "TLSv1.3":
+                ctx.minimum_version = ssl.TLSVersion.TLSv1_3
+                ctx.maximum_version = ssl.TLSVersion.TLSv1_3
+            else:
+                _logger.error('Invalid tls version: %s', tls_version)
+
+        return ctx
